#!/bin/sh
# -*- tcl -*-
# The next line is executed by /bin/sh, but not tcl \
exec tclsh "$0" ${1+"$@"}

################################################################################

# The real thing starts on line 66

################################################################################

set PI [expr {2*acos(0)}]

# Helper function copy/paste from wiki.tcl.tk

proc range {start cutoff finish {step 1}} {
        # If "start" and "finish" aren't integers, do nothing:
        if {[string is integer -strict $start] == 0 || [string is\
                integer -strict $finish] == 0} {
                error "range: Range must contain two integers"
        }

        # "Step" has to be an integer too, and
        # no infinite loops that go nowhere are allowed:
        if {$step == 0 || [string is integer -strict $step] == 0} {
                error "range: Step must be an integer other than zero"
        }

        # Does the range include the last number?
        switch $cutoff {
                "to" {set inclu 1}
                "no" {set inclu 0}
                default {
                        error "range: Use \"to\" for an inclusive\
                        range, or \"no\" for a noninclusive range"
                }
        }

        # Is the range ascending or descending (or neither)?
        set up [expr {$start <= $finish}]

        # If range is descending and step is positive but
        # doesn't have a "+" sign, change step to negative:
        if {$up == 0 && $step > 0 && [string first "+" $start] != 0} {
                set step [expr $step * -1]
        }

        # Initialize list variable and identify
        # class of integer range:
        set ranger [list]

        set op [lindex {> >= < <=} [expr {($up << 1) + $inclu}]]

        # Generate a list containing the
        # specified range of integers:
        for {set i $start} "\$i $op $finish" {incr i $step} {
                lappend ranger $i
        }
        return $ranger
}

################################################################################

# Header

puts {
<gear>
   <detectors>

      <detector name="TPC" geartype="TPCParameters">

         <maxDriftLength value="600.0"/>
         <coordinateType value="polar"/>

         <modules moduleIDStartCount="0">
}

# Modules

set module_template {
            <module >
               <readoutFrequency value ="$readoutFrequency"/>

               <PadRowLayout2D type ="FixedPadSizeDiskLayout"
                  rMin       = "$rMin"
                  rMax       = "$rMax"
                  phiMax     = "$phiMax"
                  padHeight  = "$padHeight"
                  padWidth   = "$padWidth"
                  maxRow     = "$maxRow"
                  padGap     = "$padGap">
               </PadRowLayout2D>

               <enlargeActiveAreaBy value="$enlargeActiveAreaBy"/>

               <angle value="$angle"/>
               <offset x_r="$x_r" y_phi ="$y_phi"/>
            </module>
}

#set module_template {
#            <module >
#               <readoutFrequency value ="$readoutFrequency"/>
#
#               <PadRowLayout2D type ="FixedPadAngleDiskLayout"
#                  rMin       = "$rMin"
#                  rMax       = "$rMax"
#                  phiMin     = "$phiMin"
#                  phiMax     = "$phiMax"
#                  nRow       = "$nRow"
#                  nPadsInRow = "$nPadsInRow">
#               </PadRowLayout2D>
#
#               <enlargeActiveAreaBy value="$enlargeActiveAreaBy"/>
#
#               <angle value="$angle"/>
#               <offset x_r="$x_r" y_phi ="$y_phi"/>
#            </module>
#}

foreach {nCircle nModules} {
   0 10
   1 16
   2 22
   3 28
} {
   
   set moduleHeight 300.
   
   # Each module has the dimensions $moduleHeight x (2*PI / $nModules) [r x phi]
   # The active extent can be set smaller by specifying the margins that go
   # inwards. Another way of saying, the distance between active extents are
   # 2*sideMargin in the phi direction and 2*rMargin in the r direction.
   
   set sideMargin 10.
   set rMargin    20.

   # enlargeActiveAreaBy is only used for drawing the module outline.
   # It has no influence on geometry, unlike sideMargin and rMargin.
   
   set enlargeActiveAreaBy 1.

   foreach nModule [range 1 to $nModules] {
      #set PadRowLayout2D   FixedPadAngleDiskLayout
      set readoutFrequency 40000000

      set rAvg [expr {550. + $moduleHeight*$nCircle}]
      set rMin [expr {$rAvg - $moduleHeight / 2 + $rMargin}]
      set rMax [expr {$rAvg + $moduleHeight / 2 - $rMargin}]

      # angle = s / radius, where s is the path length on the circle.
      set phiMin 0.
      set phiMax [expr {
         2. * $PI / $nModules
         - 2. * $sideMargin / ($rAvg - $moduleHeight / 2)
      }]

      set padHeight 6.5
      set padWidth  1

      set maxRow [expr {floor(($rMax - $rMin) / $padHeight)}]
      set padGap 0

      set nRow       $maxRow
      set nPadsInRow [expr {ceil($phiMax / ($padWidth / $rAvg))}]

      # offsets

      set angle [expr {
         2.*$PI * ($nModule - 1) / $nModules
         + 2.*$PI / $nModules / ($nCircle + 1)
      }]

      set x_r   0
      set y_phi 0

      #

      puts [subst $module_template]
   }
}

# Footer

puts {
         </modules>

      </detector>

   </detectors>
</gear>
}
