#ifndef __TBBQEVENTMANAGER_H__
#define __TBBQEVENTMANAGER_H__

//------------------------------------------------------------------------------
// Ref: http://root.cern.ch/root/html/tutorials/eve/alice_vsd.C.html
//------------------------------------------------------------------------------

// C++

#include <string>

// ROOT

class TGTextEntry;

#include "TEveEventManager.h"

#include "RQ_OBJECT.h"

// LCIO

namespace EVENT {
   class LCEvent;
}

namespace IO {
   class LCReader;
}

//

class VisTPC;

/*!
 *  \brief
 *  A derived class of TEveEventManager that governs the navigation through
 *  events.
 *
 *  For more information consult the pages on TEve.
 *
 *  @author Ching Bon Lam
 */

class TBBQEventManager : public TEveEventManager {
   RQ_OBJECT("TBBQEventManager")
public:
   VisTPC*         fVisTPC;
   IO::LCReader*   fLCReader;
   std::string     fCurrentFile;
   EVENT::LCEvent* fCurrentEvent;

   TGTextEntry*    fRunNumberEntry;
   TGTextEntry*    fEventNumberEntry;

   int             fFirstEventNumber;
   int             fLastEventNumber;

public:
   TBBQEventManager(VisTPC*& visTPC);

   //---------------------------------------------------------------------------
   /// @name Getter
   //@{

   EVENT::LCEvent* getCurrentEvent() {return fCurrentEvent;}

   //@}

   /// \brief
   /// Read LCIO data file.
   void readFile(const std::string& filePath, bool clearEventCollections = true);

   //---------------------------------------------------------------------------
   /// @name Event navigation and handling
   //@{

   void DropEvent();
   void GotoEvent(int eventNumber);
   void GotoEvent(const char*);
   void NextEvent();
   void PrevEvent();
   void ReloadEvent();

   //@}

   //---------------------------------------------------------------------------
   /// @name Event handlers (event in the software sense)
   //@{

   void handleEventEntry();
   void handleRunEntry();

   //@}

   //---------------------------------------------------------------------------
   /// @name Signals
   //@{

   void EventChanged();
   void EventChanged(int);         //*SIGNAL*
   void EventChanged(const char*); //*SIGNAL*

   void RunChanged();
   void RunChanged(int);           //*SIGNAL*
   void RunChanged(const char*);   //*SIGNAL*

   //@}

protected:

private:
   ClassDef(TBBQEventManager, 1);
};

#endif
