#ifndef __TBBQMANAGER_H__
#define __TBBQMANAGER_H__

// C++

#include <string>
#include <map>

// ROOT

class TObject;
class TSystem;

class TGObject;
class TGListBox;

class TEveListTreeItem;
#include <TEveManager.h>

// GEAR

namespace gear {
   class GearMgr;
}

// LCIO

namespace EVENT {
   class LCEvent;
}

namespace IO {
   class LCReader;
}

// VisTPC

class VisTPC;
class TBBQEventManager;

/*!
 *  \brief
 *  A derived class of TEveManager tailored to the needs of BBQ.
 *
 *  It provides extra GUI elements specific to BBQ.
 *
 *  For more information consult the pages on TEve.
 *
 *  @author Ching Bon Lam
 */

class TBBQManager : public TEveManager {
public:

   gear::GearMgr* fGearMgr;
   
   std::string    fCurrentGEARFile;

   VisTPC*        fVisTPC;

   TGListBox*     fHitsCollections;
   TGListBox*     fPulsesCollections;
   TGListBox*     fTracksCollections;

   std::map<std::string, TGObject*> fGUI;

public:

   TBBQManager(UInt_t  w, UInt_t  h, Bool_t  map_window = kTRUE, Option_t* opt = "FI");
   ~TBBQManager();

   static TBBQManager* Create(Bool_t map_window = kTRUE, Option_t* opt = "FIV");
   static void         Terminate();

   virtual TBBQEventManager* GetCurrentEvent() {return (TBBQEventManager*) gEve->GetCurrentEvent();}

   //---------------------------------------------------------------------------
   /// @name Getters
   
   //@{

   TGListBox*     getHitsCollections()   {return fHitsCollections;}
   TGListBox*     getPulsesCollections() {return fPulsesCollections;}
   TGListBox*     getTracksCollections() {return fTracksCollections;}
   VisTPC*        getVisTPC()            {return fVisTPC;}
   gear::GearMgr* getGearMgr()           {return fGearMgr;}
   
   //@}

   //---------------------------------------------------------------------------
   /*! @name Actions
    */

   void clearEventCollections();
   void displayEventCollections(const EVENT::LCEvent* event);

   void clearEventScene();
   void clearGlobalScene();

   //

   std::string dialogOpenFile(const char** fileTypes);
   void dialogOpenGEARFile();
   void dialogOpenLCIOFile();

   void importGEARFile(const std::string& filePath);
   void importLCIOFile(const std::string& filePath);

   void updateGUIState();
   
   //@}

   //---------------------------------------------------------------------------
   /*! @name Event handlers
    */
   
   //@{

   void handleCheckbutton(const char *);
   void handleDriftVelocityEntry(const char *text);
   void handleMaxDriftLengthEntry(const char *text);
   void handleReadoutFrequencyEntry(const char *text);

   void slotMenu(Int_t menuId);
   void slotTreeItemChecked(TEveListTreeItem* item, Bool_t check);
   
   //@}

protected:

private:
   ClassDef(TBBQManager, 1);
};

R__EXTERN TBBQManager* gBBQ;

#endif
